// EndSession.h: interface for the CEndSession class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ENDSESSION_H__8EF69441_F89F_11D4_B808_C7C80AA9BE1E__INCLUDED_)
#define AFX_ENDSESSION_H__8EF69441_F89F_11D4_B808_C7C80AA9BE1E__INCLUDED_

/* Written by Amir Israeli November 2000,
   You may use this class , modify it , but dont remove this header.
   israelaq@walla.co.il  http://amir-israeli.tripod.com */

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


/* This class is part of the Hook framework which will be offered
 in several stages in my site during 2001 
 (still needs documentation) 

 CEndSession class : This class manages situation of end session
 : when some application calls ExitWindows (whenever the situation is ... for example
 : when installing new hardware...) it sends all top level windows (or apps) in the system
 the message :WM_QUERYENDSESSION . an application should respond
 to that message : save its data. The default implementation in DefWindowProc
 in insufficient (return TRUE to permit) in case that an app has some critical data
 so it has to save data.
 How to use : 
 1) Override this class and override virtual functions :
 2) Create session manager (it actually hooks the main wnd)
 3) instantiate the class as a member variable in CFrameWnd derived class
 and call Create inside the function of OnCreate handler :
 AFTER the CFrameWnd was created.

  class CMySessionMgr : public CEndSession 
  {
	protected:
	
	// of you permit to end session no need to override this function
	// because CEndSession permits it.
	BOOL OnPermitEndSession() {  return FALSE; 	}
	virtual void OnSessionEnding() {
		// I allow continuation of boot process  but only after saving
		// my data ... (use can do it asynchonously and start the process 
		//of unloading modules in OnPermitEndSession but you'll take the risk of need to
		reload them if ExitWindows (boot/reset...) is canceled)
		CDocument *pDoc = GetDocument(); // some function to get document
		// or place where data is stored : no such function in CFrameWnd so 
		// you build one or use you view to get it. 
		CDocument::OnSaveDocument(...) // save data
		// all the process of saving may be done here
	}
	virtual void OnSessionNotEnded() {
		// do what you have to do : If you want to shoot , shoot dont talk! 
		// (Its really late).  If you did something (unloaded something) when 
		// OnPermitEndSession was called its time that you reload it back 
		// because system doesnt going to be unloaded.
	}
  };


// Example for instantiation of class:
	class CMainFrame : public CFrameWnd
	{
		...

		protected:
			CMySessionMgr m_sessionMgr;

	};

	// in cpp file
	
	int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
	{
		if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
			return -1;

		// check responce : assert or verify responce
	    ASSERT(m_sessionMgr.Create());
		// A better approach is to : VERIFY(m_sessionMgr.Create());

		// creating toolbars .... status bar etc ...
		// ...

		return 0;
	}

*/

#include "Hook.h"

// See .cpp file for description of class and how to use it

class CEndSession : protected CHook  
{
public:
	CEndSession();
	virtual ~CEndSession();
	
	// Create functions : (second with a better descriptive name)
	BOOL Create();
	BOOL CreateEndSessionMgr() { return Create(); }
	BOOL IsHooked() { return CHook::IsHooked(); }

	/* OnPermitEndSession : Override this function to permit or not ExitWindows process
	from your side : if you allow : return TRUE. But - if not (some critical condition is happenning
	or you're simply offended...) return FALSE. WindowProc2 doesn't send this message to 
	DefWindowProc which by default return TRUE */
	virtual BOOL OnPermitEndSession() { return TRUE; }
	virtual void OnSessionEnding();
	virtual void OnSessionNotEnded();

protected:
	// window proc that hooks End Session messages
	virtual LRESULT WindowProc2(UINT msg, WPARAM wp, LPARAM lp);

};

#endif // !defined(AFX_ENDSESSION_H__8EF69441_F89F_11D4_B808_C7C80AA9BE1E__INCLUDED_)
